#include "win32util_console.h"
#include <stdarg.h>
#include <stdio.h>

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

Win32Util::Console::Console() {

	// Set inited to false
	mInited=false;
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

Win32Util::Console::~Console() {

	// If window is open, close
	if (mInited) close();
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

void Win32Util::Console::open(const int pWidth, const int pHeight) {

	mWidth  = pWidth;
	mHeight = pHeight;

	// Opens a console window
	AllocConsole();
	mHandle = GetStdHandle( STD_OUTPUT_HANDLE );

	// If pWidth and pHeight is specified
	if (pWidth*pHeight>0) {

		// Set up a small rect
		SMALL_RECT rect={0,0,pWidth,pHeight};

		// Change size and 'absolue' the window (fixed size)
		SetConsoleWindowInfo(mHandle, true, &rect);

		// Set buffer equal to size so that no scrollbars appears
		COORD coord={pWidth, pHeight};
		SetConsoleScreenBufferSize(mHandle, coord);
	}

	// Hide cursor as default action
	hideCursor();
	setCursorSize(100);

	// Set default color
	setColor();

	// Set inited to true (since the windowed is opened)
	mInited=true;
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

void Win32Util::Console::close() {

	// Close the window
	FreeConsole();

	// Set inited to false (since the windowed is closed)
	mInited=false;
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

void Win32Util::Console::clear(const int pRows) {
	
	int rows = pRows;
	if (pRows==-1) rows=mHeight;

	// Go to upper left corner
	locate(0,0);

	// Make a string full of spaces equal the size of the window
	char* empty = new char[mWidth];
	memset(empty, ' ', mWidth);

	// Write the string to the console
	DWORD count;
	for (int i=0; i<=rows; i++) {
		WriteConsole(mHandle, empty, mWidth, &count, NULL);
	}
	
	// Clean up
	delete empty;

	// Go back to top left corner
	locate(0,0);
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

void Win32Util::Console::showCursor() {

	// Set variables
	mCCI.bVisible=TRUE;
	
	// Set structure
	SetConsoleCursorInfo(mHandle, &mCCI);
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

void Win32Util::Console::hideCursor() {

	// Set variables
	mCCI.bVisible=FALSE;

	// Set structure
	SetConsoleCursorInfo(mHandle, &mCCI);
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

void Win32Util::Console::setTitle(const char* pTitle) {

	// Set title
	SetConsoleTitle(pTitle);
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

void Win32Util::Console::setCursorSize(const int pSize) {

	// Set variable
	mCCI.dwSize= pSize;
	
	// Set structure
	SetConsoleCursorInfo(mHandle, &mCCI);
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

void Win32Util::Console::setColor(const int pForegroundColor, const int pBackgroundColor) {

	// Set attribute (combine colors)
	WORD attribute;
	attribute = pForegroundColor | (pBackgroundColor<<4);

	// Set color
	SetConsoleTextAttribute(mHandle, attribute);
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

void Win32Util::Console::locate(const int pX, const int pY) {

	// Temporary
	COORD coord={pX,pY};

	// Set location
	SetConsoleCursorPosition(mHandle, coord);
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

void Win32Util::Console::write(const char* pMessage, ...) {

	// Bind into one big string
	char msg[1024];		
	va_list args;
	va_start(args,pMessage);
	vsprintf(msg,pMessage,args);
	
	// Write to console
	DWORD count;
	WriteConsole(mHandle, msg, strlen(msg), &count, NULL);
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

void Win32Util::Console::write(const int pX, const int pY, const char* pMessage, ...) {

	// Move cursor
	locate(pX, pY);

	// Bind into one big string
	char msg[1024];		
	va_list args;
	va_start(args,pMessage);
	vsprintf(msg,pMessage,args);
	
	// Write to console
	DWORD count;
	WriteConsole(mHandle, msg, strlen(msg), &count, NULL);
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

